Revert "gdb/gdbserver: share x86/linux tdesc caching"
This reverts commit 198ff6ff819c240545f9fc68b39636fd376d4ba9.
diff --git a/gdb/amd64-linux-tdep.c b/gdb/amd64-linux-tdep.c
index e149034..7e0900d 100644
--- a/gdb/amd64-linux-tdep.c
+++ b/gdb/amd64-linux-tdep.c
@@ -1579,6 +1579,37 @@
return 0;
}
+const target_desc *
+amd64_linux_read_description (uint64_t xcr0_features_bit, bool is_x32)
+{
+ static target_desc *amd64_linux_tdescs \
+ [2/*AVX*/][2/*MPX*/][2/*AVX512*/][2/*PKRU*/] = {};
+ static target_desc *x32_linux_tdescs \
+ [2/*AVX*/][2/*AVX512*/][2/*PKRU*/] = {};
+
+ target_desc **tdesc;
+
+ if (is_x32)
+ {
+ tdesc = &x32_linux_tdescs[(xcr0_features_bit & X86_XSTATE_AVX) ? 1 : 0 ]
+ [(xcr0_features_bit & X86_XSTATE_AVX512) ? 1 : 0]
+ [(xcr0_features_bit & X86_XSTATE_PKRU) ? 1 : 0];
+ }
+ else
+ {
+ tdesc = &amd64_linux_tdescs[(xcr0_features_bit & X86_XSTATE_AVX) ? 1 : 0]
+ [(xcr0_features_bit & X86_XSTATE_MPX) ? 1 : 0]
+ [(xcr0_features_bit & X86_XSTATE_AVX512) ? 1 : 0]
+ [(xcr0_features_bit & X86_XSTATE_PKRU) ? 1 : 0];
+ }
+
+ if (*tdesc == NULL)
+ *tdesc = amd64_create_target_description (xcr0_features_bit, is_x32,
+ true, true);
+
+ return *tdesc;
+}
+
/* Get Linux/x86 target description from core dump. */
static const struct target_desc *
diff --git a/gdb/i386-linux-tdep.c b/gdb/i386-linux-tdep.c
index ed14422..f5f7a36 100644
--- a/gdb/i386-linux-tdep.c
+++ b/gdb/i386-linux-tdep.c
@@ -681,12 +681,29 @@
layout) != 0;
}
-/* See nat/x86-linux-tdesc.h. */
+/* See i386-linux-tdep.h. */
-void
-x86_linux_post_init_tdesc (target_desc *tdesc, bool is_64bit)
+const struct target_desc *
+i386_linux_read_description (uint64_t xcr0)
{
- /* Nothing. */
+ if (xcr0 == 0)
+ return NULL;
+
+ static struct target_desc *i386_linux_tdescs \
+ [2/*X87*/][2/*SSE*/][2/*AVX*/][2/*MPX*/][2/*AVX512*/][2/*PKRU*/] = {};
+ struct target_desc **tdesc;
+
+ tdesc = &i386_linux_tdescs[(xcr0 & X86_XSTATE_X87) ? 1 : 0]
+ [(xcr0 & X86_XSTATE_SSE) ? 1 : 0]
+ [(xcr0 & X86_XSTATE_AVX) ? 1 : 0]
+ [(xcr0 & X86_XSTATE_MPX) ? 1 : 0]
+ [(xcr0 & X86_XSTATE_AVX512) ? 1 : 0]
+ [(xcr0 & X86_XSTATE_PKRU) ? 1 : 0];
+
+ if (*tdesc == NULL)
+ *tdesc = i386_create_target_description (xcr0, true, false);
+
+ return *tdesc;
}
/* Get Linux/x86 target description from core dump. */
@@ -699,10 +716,7 @@
/* Linux/i386. */
x86_xsave_layout layout;
uint64_t xcr0 = i386_linux_core_read_xsave_info (abfd, layout);
-
- const struct target_desc *tdesc;
- if (xcr0 != 0)
- tdesc = i386_linux_read_description (xcr0);
+ const struct target_desc *tdesc = i386_linux_read_description (xcr0);
if (tdesc != NULL)
return tdesc;
diff --git a/gdb/nat/x86-linux-tdesc.c b/gdb/nat/x86-linux-tdesc.c
index c438dfa..be7014d 100644
--- a/gdb/nat/x86-linux-tdesc.c
+++ b/gdb/nat/x86-linux-tdesc.c
@@ -38,8 +38,6 @@
#include <sys/user.h>
#include <sys/user.h>
-#ifndef IN_PROCESS_AGENT
-
/* See nat/x86-linux-tdesc.h. */
const target_desc *
@@ -124,288 +122,3 @@
gdb_assert_not_reached ("failed to return tdesc");
}
-
-#endif /* !IN_PROCESS_AGENT */
-
-
-
-/* A structure used to describe a single cpu feature that might, or might
- not, be checked for when creating a target description for one of i386,
- amd64, or x32. */
-
-struct x86_tdesc_feature {
- /* The cpu feature mask. This is a mask against an xcr0 value. */
- uint64_t feature;
-
- /* Is this feature checked when creating an i386 target description. */
- bool is_i386;
-
- /* Is this feature checked when creating an amd64 target description. */
- bool is_amd64;
-
- /* Is this feature checked when creating an x32 target description. */
- bool is_x32;
-};
-
-/* A constant table that describes all of the cpu features that are
- checked when building a target description for i386, amd64, or x32. */
-
-static constexpr x86_tdesc_feature x86_linux_all_tdesc_features[] = {
- /* Feature, i386, amd64, x32. */
- { X86_XSTATE_PKRU, true, true, true },
- { X86_XSTATE_AVX512, true, true, true },
- { X86_XSTATE_AVX, true, true, true },
- { X86_XSTATE_MPX, true, true, false },
- { X86_XSTATE_SSE, true, false, false },
- { X86_XSTATE_X87, true, false, false }
-};
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an i386 target description. */
-
-static constexpr uint64_t
-x86_linux_i386_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_i386)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an amd64 target description. */
-
-static constexpr uint64_t
-x86_linux_amd64_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_amd64)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an x32 target description. */
-
-static constexpr uint64_t
-x86_linux_x32_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_x32)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an i386 target description. */
-
-static constexpr int
-x86_linux_i386_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_i386)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an amd64 target description. */
-
-static constexpr int
-x86_linux_amd64_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_amd64)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an x32 target description. */
-
-static constexpr int
-x86_linux_x32_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_x32)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-#ifdef IN_PROCESS_AGENT
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_amd64_ipa_tdesc_count ()
-{
- return x86_linux_amd64_tdesc_count ();
-}
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_x32_ipa_tdesc_count ()
-{
- return x86_linux_x32_tdesc_count ();
-}
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_i386_ipa_tdesc_count ()
-{
- return x86_linux_i386_tdesc_count ();
-}
-
-#endif /* IN_PROCESS_AGENT */
-
-/* Convert an xcr0 value into an integer. The integer will be passed to
- the in-process-agent where it will then be passed to
- x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
-
-int
-x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0)
-{
- /* The following table shows which features are checked for when creating
- the target descriptions (see nat/x86-linux-tdesc.c), the feature order
- represents the bit order within the generated index number.
-
- i386 | x87 sse mpx avx avx512 pkru
- amd64 | mpx avx avx512 pkru
- i32 | avx avx512 pkru
-
- The features are ordered so that for each mode (i386, amd64, i32) the
- generated index will form a continuous range. */
-
- int idx = 0;
-
- for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
- {
- if ((xcr0 & x86_linux_all_tdesc_features[i].feature) != 0)
- idx |= (1 << i);
- }
-
- return idx;
-}
-
-
-#ifdef IN_PROCESS_AGENT
-
-/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
- into an xcr0 value which can then be used to create a target
- description. */
-
-uint64_t
-x86_linux_tdesc_idx_to_xcr0 (int idx)
-{
- uint64_t xcr0 = 0;
-
- for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
- {
- if ((idx & (1 << i)) != 0)
- xcr0 |= x86_linux_all_tdesc_features[i].feature;
- }
-
- return xcr0;
-}
-
-#endif /* IN_PROCESS_AGENT */
-
-#if defined __i386__ || !defined IN_PROCESS_AGENT
-
-/* A cache of all possible i386 target descriptions. */
-
-static struct target_desc *i386_tdescs[x86_linux_i386_tdesc_count ()] = { };
-
-/* See nat/x86-linux-tdesc.h. */
-
-const struct target_desc *
-i386_linux_read_description (uint64_t xcr0)
-{
- xcr0 &= x86_linux_i386_tdesc_feature_mask ();
- int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
-
- gdb_assert (idx >= 0 && idx < x86_linux_i386_tdesc_count ());
-
- target_desc **tdesc = &i386_tdescs[idx];
-
- if (*tdesc == nullptr)
- {
- *tdesc = i386_create_target_description (xcr0, true, false);
-
- x86_linux_post_init_tdesc (*tdesc, false);
- }
-
- return *tdesc;
-}
-#endif
-
-#ifdef __x86_64__
-
-/* A cache of all possible amd64 target descriptions. */
-
-static target_desc *amd64_tdescs[x86_linux_amd64_tdesc_count ()] = { };
-
-/* A cache of all possible x32 target descriptions. */
-
-static target_desc *x32_tdescs[x86_linux_x32_tdesc_count ()] = { };
-
-/* See nat/x86-linux-tdesc.h. */
-
-const struct target_desc *
-amd64_linux_read_description (uint64_t xcr0, bool is_x32)
-{
- if (is_x32)
- xcr0 &= x86_linux_x32_tdesc_feature_mask ();
- else
- xcr0 &= x86_linux_amd64_tdesc_feature_mask ();
-
- int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
-
- if (is_x32)
- gdb_assert (idx >= 0 && idx < x86_linux_x32_tdesc_count ());
- else
- gdb_assert (idx >= 0 && idx < x86_linux_amd64_tdesc_count ());
-
- target_desc **tdesc = nullptr;
-
- if (is_x32)
- tdesc = &x32_tdescs[idx];
- else
- tdesc = &amd64_tdescs[idx];
-
- if (*tdesc == nullptr)
- {
- *tdesc = amd64_create_target_description (xcr0, is_x32, true, true);
-
- x86_linux_post_init_tdesc (*tdesc, true);
- }
- return *tdesc;
-}
-
-#endif
diff --git a/gdbserver/Makefile.in b/gdbserver/Makefile.in
index bd6f68e..c712089 100644
--- a/gdbserver/Makefile.in
+++ b/gdbserver/Makefile.in
@@ -519,10 +519,6 @@
$(IPAGENT_COMPILE) $<
$(POSTCOMPILE)
-nat/%-ipa.o: ../gdb/nat/%.c
- $(IPAGENT_COMPILE) $<
- $(POSTCOMPILE)
-
%-ipa.o: ../gdb/%.c
$(IPAGENT_COMPILE) -x c++ $<
$(POSTCOMPILE)
diff --git a/gdbserver/configure.srv b/gdbserver/configure.srv
index 36a457f..7a2702d 100644
--- a/gdbserver/configure.srv
+++ b/gdbserver/configure.srv
@@ -116,7 +116,6 @@
srv_linux_btrace=yes
ipa_obj="linux-i386-ipa.o linux-x86-tdesc-ipa.o"
ipa_obj="${ipa_obj} arch/i386-ipa.o"
- ipa_obj="${ipa_obj} nat/x86-linux-tdesc-ipa.o"
;;
i[34567]86-*-mingw*) srv_regobj=""
srv_tgtobj="x86-low.o nat/x86-dregs.o win32-low.o"
@@ -381,7 +380,6 @@
srv_linux_btrace=yes
ipa_obj="linux-amd64-ipa.o linux-x86-tdesc-ipa.o"
ipa_obj="${ipa_obj} arch/amd64-ipa.o"
- ipa_obj="${ipa_obj} nat/x86-linux-tdesc-ipa.o"
;;
x86_64-*-mingw*) srv_regobj=""
srv_tgtobj="x86-low.o nat/x86-dregs.o"
diff --git a/gdbserver/linux-amd64-ipa.cc b/gdbserver/linux-amd64-ipa.cc
index afa6aab..42de7d2 100644
--- a/gdbserver/linux-amd64-ipa.cc
+++ b/gdbserver/linux-amd64-ipa.cc
@@ -21,6 +21,7 @@
#include "server.h"
#include <sys/mman.h>
#include "tracepoint.h"
+#include "linux-x86-tdesc.h"
#include "gdbsupport/x86-xstate.h"
#include "nat/x86-linux-tdesc.h"
diff --git a/gdbserver/linux-i386-ipa.cc b/gdbserver/linux-i386-ipa.cc
index f6d26f7..246bcb9 100644
--- a/gdbserver/linux-i386-ipa.cc
+++ b/gdbserver/linux-i386-ipa.cc
@@ -21,6 +21,7 @@
#include "server.h"
#include <sys/mman.h>
#include "tracepoint.h"
+#include "linux-x86-tdesc.h"
#include "gdbsupport/x86-xstate.h"
#include "nat/x86-linux-tdesc.h"
diff --git a/gdbserver/linux-x86-low.cc b/gdbserver/linux-x86-low.cc
index 7134d20..6dbefdb 100644
--- a/gdbserver/linux-x86-low.cc
+++ b/gdbserver/linux-x86-low.cc
@@ -47,6 +47,7 @@
#include "nat/linux-nat.h"
#include "nat/x86-linux.h"
#include "nat/x86-linux-dregs.h"
+#include "linux-x86-tdesc.h"
#include "nat/x86-linux-tdesc.h"
#ifdef __x86_64__
diff --git a/gdbserver/linux-x86-tdesc.cc b/gdbserver/linux-x86-tdesc.cc
index 39d1d51..87cf368 100644
--- a/gdbserver/linux-x86-tdesc.cc
+++ b/gdbserver/linux-x86-tdesc.cc
@@ -19,19 +19,292 @@
#include "server.h"
#include "tdesc.h"
+#include "linux-x86-tdesc.h"
+#include "arch/i386.h"
+#include "gdbsupport/x86-xstate.h"
+#ifdef __x86_64__
+#include "arch/amd64.h"
+#endif
#include "x86-tdesc.h"
#include "nat/x86-linux-tdesc.h"
-/* See nat/x86-linux-tdesc.h. */
+/* A structure used to describe a single cpu feature that might, or might
+ not, be checked for when creating a target description for one of i386,
+ amd64, or x32. */
-void
-x86_linux_post_init_tdesc (target_desc *tdesc, bool is_64bit)
+struct x86_tdesc_feature {
+ /* The cpu feature mask. This is a mask against an xcr0 value. */
+ uint64_t feature;
+
+ /* Is this feature checked when creating an i386 target description. */
+ bool is_i386;
+
+ /* Is this feature checked when creating an amd64 target description. */
+ bool is_amd64;
+
+ /* Is this feature checked when creating an x32 target description. */
+ bool is_x32;
+};
+
+/* A constant table that describes all of the cpu features that are
+ checked when building a target description for i386, amd64, or x32. */
+
+static constexpr x86_tdesc_feature x86_linux_all_tdesc_features[] = {
+ /* Feature, i386, amd64, x32. */
+ { X86_XSTATE_PKRU, true, true, true },
+ { X86_XSTATE_AVX512, true, true, true },
+ { X86_XSTATE_AVX, true, true, true },
+ { X86_XSTATE_MPX, true, true, false },
+ { X86_XSTATE_SSE, true, false, false },
+ { X86_XSTATE_X87, true, false, false }
+};
+
+/* Return a compile time constant which is a mask of all the cpu features
+ that are checked for when building an i386 target description. */
+
+static constexpr uint64_t
+x86_linux_i386_tdesc_feature_mask ()
{
-#ifdef __x86_64__
- if (is_64bit)
- init_target_desc (tdesc, amd64_expedite_regs);
- else
-#endif
- init_target_desc (tdesc, i386_expedite_regs);
+ uint64_t mask = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_i386)
+ mask |= entry.feature;
+
+ return mask;
}
+/* Return a compile time constant which is a mask of all the cpu features
+ that are checked for when building an amd64 target description. */
+
+static constexpr uint64_t
+x86_linux_amd64_tdesc_feature_mask ()
+{
+ uint64_t mask = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_amd64)
+ mask |= entry.feature;
+
+ return mask;
+}
+
+/* Return a compile time constant which is a mask of all the cpu features
+ that are checked for when building an x32 target description. */
+
+static constexpr uint64_t
+x86_linux_x32_tdesc_feature_mask ()
+{
+ uint64_t mask = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_x32)
+ mask |= entry.feature;
+
+ return mask;
+}
+
+/* Return a compile time constant which is a count of the number of cpu
+ features that are checked for when building an i386 target description. */
+
+static constexpr int
+x86_linux_i386_tdesc_count ()
+{
+ uint64_t count = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_i386)
+ ++count;
+
+ gdb_assert (count > 0);
+
+ return (1 << count);
+}
+
+/* Return a compile time constant which is a count of the number of cpu
+ features that are checked for when building an amd64 target description. */
+
+static constexpr int
+x86_linux_amd64_tdesc_count ()
+{
+ uint64_t count = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_amd64)
+ ++count;
+
+ gdb_assert (count > 0);
+
+ return (1 << count);
+}
+
+/* Return a compile time constant which is a count of the number of cpu
+ features that are checked for when building an x32 target description. */
+
+static constexpr int
+x86_linux_x32_tdesc_count ()
+{
+ uint64_t count = 0;
+
+ for (const auto &entry : x86_linux_all_tdesc_features)
+ if (entry.is_x32)
+ ++count;
+
+ gdb_assert (count > 0);
+
+ return (1 << count);
+}
+
+#ifdef IN_PROCESS_AGENT
+
+/* See linux-x86-tdesc.h. */
+
+int
+x86_linux_amd64_ipa_tdesc_count ()
+{
+ return x86_linux_amd64_tdesc_count ();
+}
+
+/* See linux-x86-tdesc.h. */
+
+int
+x86_linux_x32_ipa_tdesc_count ()
+{
+ return x86_linux_x32_tdesc_count ();
+}
+
+/* See linux-x86-tdesc.h. */
+
+int
+x86_linux_i386_ipa_tdesc_count ()
+{
+ return x86_linux_i386_tdesc_count ();
+}
+
+#endif /* IN_PROCESS_AGENT */
+
+/* Convert an xcr0 value into an integer. The integer will be passed to
+ the in-process-agent where it will then be passed to
+ x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
+
+int
+x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0)
+{
+ /* The following table shows which features are checked for when creating
+ the target descriptions (see nat/x86-linux-tdesc.c), the feature order
+ represents the bit order within the generated index number.
+
+ i386 | x87 sse mpx avx avx512 pkru
+ amd64 | mpx avx avx512 pkru
+ i32 | avx avx512 pkru
+
+ The features are ordered so that for each mode (i386, amd64, i32) the
+ generated index will form a continuous range. */
+
+ int idx = 0;
+
+ for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
+ {
+ if ((xcr0 & x86_linux_all_tdesc_features[i].feature) != 0)
+ idx |= (1 << i);
+ }
+
+ return idx;
+}
+
+
+#ifdef IN_PROCESS_AGENT
+
+/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
+ into an xcr0 value which can then be used to create a target
+ description. */
+
+uint64_t
+x86_linux_tdesc_idx_to_xcr0 (int idx)
+{
+ uint64_t xcr0 = 0;
+
+ for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
+ {
+ if ((idx & (1 << i)) != 0)
+ xcr0 |= x86_linux_all_tdesc_features[i].feature;
+ }
+
+ return xcr0;
+}
+
+#endif /* IN_PROCESS_AGENT */
+
+#if defined __i386__ || !defined IN_PROCESS_AGENT
+
+/* A cache of all possible i386 target descriptions. */
+
+static struct target_desc *i386_tdescs[x86_linux_i386_tdesc_count ()] = { };
+
+/* See nat/x86-linux-tdesc.h. */
+
+const struct target_desc *
+i386_linux_read_description (uint64_t xcr0)
+{
+ xcr0 &= x86_linux_i386_tdesc_feature_mask ();
+ int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
+
+ gdb_assert (idx >= 0 && idx < x86_linux_i386_tdesc_count ());
+
+ target_desc **tdesc = &i386_tdescs[idx];
+
+ if (*tdesc == nullptr)
+ {
+ *tdesc = i386_create_target_description (xcr0, true, false);
+
+ init_target_desc (*tdesc, i386_expedite_regs);
+ }
+
+ return *tdesc;
+}
+#endif
+
+#ifdef __x86_64__
+
+/* A cache of all possible amd64 target descriptions. */
+
+static target_desc *amd64_tdescs[x86_linux_amd64_tdesc_count ()] = { };
+
+/* A cache of all possible x32 target descriptions. */
+
+static target_desc *x32_tdescs[x86_linux_x32_tdesc_count ()] = { };
+
+/* See nat/x86-linux-tdesc.h. */
+
+const struct target_desc *
+amd64_linux_read_description (uint64_t xcr0, bool is_x32)
+{
+ if (is_x32)
+ xcr0 &= x86_linux_x32_tdesc_feature_mask ();
+ else
+ xcr0 &= x86_linux_amd64_tdesc_feature_mask ();
+
+ int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
+
+ if (is_x32)
+ gdb_assert (idx >= 0 && idx < x86_linux_x32_tdesc_count ());
+ else
+ gdb_assert (idx >= 0 && idx < x86_linux_amd64_tdesc_count ());
+
+ target_desc **tdesc = nullptr;
+
+ if (is_x32)
+ tdesc = &x32_tdescs[idx];
+ else
+ tdesc = &amd64_tdescs[idx];
+
+ if (*tdesc == nullptr)
+ {
+ *tdesc = amd64_create_target_description (xcr0, is_x32, true, true);
+
+ init_target_desc (*tdesc, amd64_expedite_regs);
+ }
+ return *tdesc;
+}
+
+#endif
diff --git a/gdbserver/linux-x86-tdesc.h b/gdbserver/linux-x86-tdesc.h
new file mode 100644
index 0000000..70456e4
--- /dev/null
+++ b/gdbserver/linux-x86-tdesc.h
@@ -0,0 +1,50 @@
+/* Low level support for x86 (i386 and x86-64), shared between gdbserver
+ and IPA.
+
+ Copyright (C) 2016-2024 Free Software Foundation, Inc.
+
+ This file is part of GDB.
+
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 3 of the License, or
+ (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program. If not, see <http://www.gnu.org/licenses/>. */
+
+#ifndef GDBSERVER_LINUX_X86_TDESC_H
+#define GDBSERVER_LINUX_X86_TDESC_H
+
+/* Convert an xcr0 value into an integer. The integer will be passed to
+ the in-process-agent where it will then be passed to
+ x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
+
+extern int x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0);
+
+#ifdef IN_PROCESS_AGENT
+
+/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
+ into an xcr0 value which can then be used to create a target
+ description. */
+
+extern uint64_t x86_linux_tdesc_idx_to_xcr0 (int idx);
+
+/* Within the in-process-agent we need to pre-initialise all of the target
+ descriptions, to do this we need to know how many target descriptions
+ there are for each different target type. These functions return the
+ target description count for the relevant target. */
+
+extern int x86_linux_amd64_ipa_tdesc_count ();
+extern int x86_linux_x32_ipa_tdesc_count ();
+extern int x86_linux_i386_ipa_tdesc_count ();
+
+
+#endif /* IN_PROCESS_AGENT */
+
+#endif /* GDBSERVER_LINUX_X86_TDESC_H */
diff --git a/gdb/nat/x86-linux-tdesc.h b/gdb/nat/x86-linux-tdesc.h
index 648fe0e..3727a8b 100644
--- a/gdb/nat/x86-linux-tdesc.h
+++ b/gdb/nat/x86-linux-tdesc.h
@@ -24,8 +24,6 @@
struct target_desc;
-#ifndef IN_PROCESS_AGENT
-
/* Return the target description for Linux thread TID.
When *HAVE_PTRACE_GETREGSET is TRIBOOL_UNKNOWN then the current value of
@@ -59,57 +57,19 @@
gdb::function_view<void (uint64_t)> xcr0_init_cb,
const char *error_msg, uint64_t *xcr0_storage);
-#endif /* !IN_PROCESS_AGENT */
-
#ifdef __x86_64__
-/* Return the AMD64 target descriptions corresponding to XCR0 and IS_X32. */
+/* Return the right amd64-linux target descriptions according to
+ XCR0_FEATURES_BIT and IS_X32. This is implemented separately in both
+ GDB and gdbserver. */
-extern const target_desc *amd64_linux_read_description (uint64_t xcr0,
- bool is_x32);
+extern const target_desc *amd64_linux_read_description
+ (uint64_t xcr0_features_bit, bool is_x32);
-#endif /* __x86_64__ */
+#endif
-/* Return the i386 target description corresponding to XCR0. */
-
+/* Return the target description according to XCR0. This is implemented
+ separately in both GDB and gdbserver. */
extern const struct target_desc *i386_linux_read_description (uint64_t xcr0);
-/* This function is called from amd64_linux_read_description and
- i386_linux_read_description after a new target description has been
- created, TDESC is the new target description, IS_64BIT will be true
- when called from amd64_linux_read_description, otherwise IS_64BIT will
- be false. If the *_linux_read_description functions found a cached
- target description then this function will not be called.
-
- Both GDB and gdbserver have their own implementations of this
- function. */
-
-extern void x86_linux_post_init_tdesc (target_desc *tdesc, bool is_64bit);
-
-/* Convert an xcr0 value into an integer. The integer will be passed to
- the in-process-agent where it will then be passed to
- x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
-
-extern int x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0);
-
-
-#ifdef IN_PROCESS_AGENT
-
-/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
- into an xcr0 value which can then be used to create a target
- description. */
-
-extern uint64_t x86_linux_tdesc_idx_to_xcr0 (int idx);
-
-/* Within the in-process-agent we need to pre-initialise all of the target
- descriptions, to do this we need to know how many target descriptions
- there are for each different target type. These functions return the
- target description count for the relevant target. */
-
-extern int x86_linux_amd64_ipa_tdesc_count ();
-extern int x86_linux_x32_ipa_tdesc_count ();
-extern int x86_linux_i386_ipa_tdesc_count ();
-
-#endif /* IN_PROCESS_AGENT */
-
#endif /* NAT_X86_LINUX_TDESC_H */